//============================================================================
// trimodel.hpp : triangle and triangle model class definitions
//  The following conversions are possible:
//    1) triangle soup <-> shared vert tris
//    2) shared vert tris <-> shared vert tris with vertex normals
//  File formats:
//    1) triangle soup:
//         # of triangles
//         list of triangles (sets of 9 floats - 3 3D points)
//    2) shared vert tris:
//         # of verts
//         list of verts (sets of 3 floats - 3D point)
//         # of tris
//         list of tris (sets of 3 ints - indices into vert array)
//    3) shared vert tris with vertex normals:
//         # of verts
//         list of verts (sets of 6 floats - 3D point and normal)
//         # of tris
//         list of tris (sets of 3 ints - indices into vert array)
//============================================================================

#ifndef TRIMODEL
#define TRIMODEL

#include "vec3f.hpp"

//----------------------------------------------------------------------------
// BASIC TRIANGLE CLASS (COMPOSED OF THREE VERTICES AND A NORMAL)
//----------------------------------------------------------------------------
class Tri
{
 public:
  Vec3f A,B,C,N;
  int Color;
};

//----------------------------------------------------------------------------
// TRIANGLE THAT "SHARES VERTICES" (VERTS ARE PTRS INTO SHARED VERTEX ARRAY)
//----------------------------------------------------------------------------
struct SharedVertTri
{
 public:
  int iA,iB,iC;           // INDICES INTO SHARED VERTEX ARRAY
  Vec3f N;                // TRIANGLE NORMAL
  int Color;
};


//----------------------------------------------------------------------------
// A SHARED VERTEX WITH A NORMAL
//----------------------------------------------------------------------------
class NormVert
{
 public:
  Vec3f V,N;
};


class SharedVertTriModel;
class SharedVertNormTriModel;


//----------------------------------------------------------------------------
// BASIC TRIANGLE MODEL COMPOSED OF AN ARRAY OF "BASIC" TRIANGLES
//----------------------------------------------------------------------------
class TriModel
{
 public:

  Tri *Tris;     // ARRAY OF "BASIC" TRIANGLES
  int NumTris;

  Vec3f Min,Max; // MIN AND MAX EXTENTS OF THE MODEL

  TriModel() {};
  TriModel(char* FileName);
  TriModel(SharedVertTriModel &SVTM);
  ~TriModel() { delete[] Tris; };
  void CalcExtents();
  void PrintToFile(char* FileName);
  void PrintToVRML(char* FileName);
  void Display();
};


//----------------------------------------------------------------------------
// TRIANGLE MODEL COMPOSED OF TRIANGLES W/ SHARED VERTICES
//----------------------------------------------------------------------------
class SharedVertTriModel
{
 public:

  Vec3f *Verts;       // SHARED VERTEX ARRAY (TRIS COMPOSED OF INDICES INTO THIS)
  int NumVerts;
  SharedVertTri *Tris; // ARRAY OF TRIS W/ SHARED VERTS
  int NumTris;

  Vec3f Min,Max; // MIN AND MAX EXTENTS OF THE MODEL

  SharedVertTriModel() {};
  SharedVertTriModel(char* FileName);
  SharedVertTriModel(TriModel &TM);
  SharedVertTriModel(SharedVertNormTriModel &SVTMN);
  ~SharedVertTriModel() { delete[] Verts; delete[] Tris; };
  void ReAllocNumVerts(int NewNumVerts);
  void CalcExtents();
  void PrintToFile(char* FileName);
  void PrintToVRML(char* FileName);
  void Display();
};


//----------------------------------------------------------------------------
// TRIANGLE MODEL COMPOSED OF TRIANGLES W/ SHARED VERTICES (w/ VERTEX NORMALS)
//----------------------------------------------------------------------------
class SharedVertNormTriModel
{
 public:

  NormVert *Verts;     // SHARED VERTEX ARRAY (TRIS COMPOSED OF INDICES INTO THIS)
  int NumVerts;
  SharedVertTri *Tris; // ARRAY OF TRIS W/ SHARED VERTS
  int NumTris;

  Vec3f Min,Max; // MIN AND MAX EXTENTS OF THE MODEL

  SharedVertNormTriModel() {};
  SharedVertNormTriModel(char* FileName);
  SharedVertNormTriModel(SharedVertTriModel &SVTM);
  ~SharedVertNormTriModel() { delete[] Verts; delete[] Tris; };
  void CalcExtents();
  void PrintToFile(char* FileName);
  void PrintToVRML(char* FileName);
  void Display();
};

#endif

